#include <QApplication>
#include <QLocale>
#include <QtGui>

#include <QAbstractTableModel>
#include <QTableView>

#include <QItemSelection>

class PaysTableModel : public QAbstractTableModel
{
    private:
        QStringList listePays;
        QStringList listeCapitales;
        enum Colonnes { Pays = 0, Capitale = 1, NbColonnes = Capitale + 1};


    public:
        PaysTableModel(const QStringList &listePays, QStringList &listeCapitales, QObject *parent = 0):QAbstractTableModel(parent)
        {if (listePays.count() > 0) this->listePays = listePays;
         if (listeCapitales.count() > 0) this->listeCapitales = listeCapitales;
        }
        ~PaysTableModel() {}
        int rowCount(const QModelIndex &parent = QModelIndex()) const
        { return parent.isValid() ? 0 : this->listePays.count(); }
        int columnCount(const QModelIndex &parent = QModelIndex()) const
        { return parent.isValid() ? 0 : NbColonnes; }
        QVariant data(const QModelIndex &index, int role) const
        {
           if (! index.isValid())
                return QVariant();
           if (((unsigned int) index.row() >= this->listePays.count()) || ((unsigned int) index.column() >= NbColonnes))
                return QVariant();
           if ((role == Qt::DisplayRole || role == Qt::EditRole))
           {
              if(index.column() == Pays)
                return this->listePays.at(index.row());
              else if(index.column() == Capitale)
                return this->listeCapitales.at(index.row());
           }
           return QVariant();
        }
        bool setData(const QModelIndex &index, const QVariant &value, int role)
        {
            if (index.isValid() && role == Qt::EditRole) {
                this->listeCapitales.replace(index.row(), value.toString());
                emit(dataChanged(index, index));
                return true;
            }
            return false;
        }
        Qt::ItemFlags flags(const QModelIndex &index) const
        {
            if (! index.isValid())
                return Qt::ItemIsEnabled;
            if (index.column() == Pays) // Un pays seulement selectionnable
            {
                return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
            }
            else if (index.column() == Capitale) // Une capitale sera éditable
            {
                return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable;
            }
            return QAbstractTableModel::flags(index);
        }

};

int main(int argc, char *argv[])
{
     QApplication app(argc, argv);
     QMainWindow  mainWindow;
     QWidget      tableWidget;

     QStringList listePays;
     listePays << "France" << "Angleterre" << "Espagne" << "Italie" << "Allemagne";

     QStringList listeCapitales;
     listeCapitales << "Paris" << "?" << "?" << "?" << "?";

     PaysTableModel *modele = new PaysTableModel(listePays, listeCapitales);

     QTableView *vueTable = new QTableView;
     vueTable->setModel(modele);

     QHBoxLayout *layout = new QHBoxLayout;
     layout->addWidget(vueTable);
     tableWidget.setLayout(layout);
     mainWindow.setCentralWidget(&tableWidget);
     mainWindow.setWindowTitle("Exemple 4");
     mainWindow.show();

     return app.exec();
}
