#include <QApplication>
#include <QtGui>
#include <QDebug>

class MyPushButton : public QPushButton
{
   Q_OBJECT
   public:
      MyPushButton( QString libelle, QWidget *parent = 0 ) : QPushButton( libelle, parent ) 
      {
         resize(120, 30);
         QPalette palette;
         palette.setColor(QPalette::Button, QColor(127,127,127));
         palette.setColor(QPalette::ButtonText, QColor(Qt::white));
         this->setPalette(palette);
      }
      void mousePressEvent(QMouseEvent *event)
      {
         if(event->button() == Qt::LeftButton)
            _dragStartPosition = event->pos();
      }
      void mouseMoveEvent(QMouseEvent *event)
      {
         if (!(event->buttons() & Qt::LeftButton))
            return;
         if ((event->pos() - _dragStartPosition).manhattanLength()
               < QApplication::startDragDistance())            
            return;

         QDrag *drag = new QDrag(this); 
         QMimeData *mimeData = new QMimeData;
         QPixmap pixmap(this->width(), this->height());  
         QPainter painter;  
         painter.begin(&pixmap);     
         painter.fillRect(pixmap.rect(), QColor(127,0,127));
         painter.setPen( QPen(Qt::black, 4) );
         painter.setFont(QFont("Courier New", 12, QFont::Bold));
         painter.drawText(pixmap.rect(), Qt::AlignCenter, "move");
         painter.end();  
      
         drag->setPixmap(pixmap);
         drag->setHotSpot(event->pos());

         mimeData->setText(this->text());
         drag->setMimeData(mimeData);

         //drag->exec(Qt::CopyAction | Qt::MoveAction);
          Qt::DropAction dropAction = drag->exec();
          
          if(dropAction == Qt::MoveAction)
             close();
      }
      QPoint dragStartPosition() {
         return _dragStartPosition;
      }
      
   private:
      QPoint _dragStartPosition;
};

class MyFrame : public QFrame 
{
   Q_OBJECT
   public:
      MyFrame( QWidget *parent = 0 ) : QFrame( parent ) 
      {
         setMinimumSize(170, 185);
         setFrameStyle(QFrame::Sunken | QFrame::StyledPanel);
         setAcceptDrops(true);
         
         btn1 = new MyPushButton("Bouton 1", this);
         btn1->move(25, 145);
         
         btn2 = new MyPushButton("Bouton 2", this);
         btn2->move(25, 45);
         
         btn1->show();
         btn2->show();
      }
      
      void dragEnterEvent(QDragEnterEvent *event)
      {
         if (event->mimeData()->hasFormat("text/plain") && event->proposedAction() == Qt::MoveAction) 
         {
            event->acceptProposedAction();
         } 
         else 
         {
            event->ignore();
         }
      }
      void dragMoveEvent(QDragMoveEvent *event)
      {
         if (event->mimeData()->hasFormat("text/plain") && event->proposedAction() == Qt::MoveAction)
         {
            MyPushButton *childBtn = static_cast<MyPushButton*>(event->source());
            if(this == childBtn->parentWidget())
                  event->ignore();
            else
                  event->acceptProposedAction();
         } else {
            event->ignore();
         }
      }
      void dropEvent(QDropEvent *event)
      {
         if (event->mimeData()->hasFormat("text/plain") && event->proposedAction() == Qt::MoveAction)
            {
            MyPushButton *childBtn = static_cast<MyPushButton*>(event->source());
            if(this != childBtn->parentWidget())
            {
               // la cible peut aussi supprimer le widget source :
               //childBtn->close();
               MyPushButton *newBtn = new MyPushButton(event->mimeData()->text(), this);
               newBtn->move(event->pos() - childBtn->dragStartPosition());
               newBtn->show();
            }

            event->acceptProposedAction();

         } else {
            event->ignore();
         }
      }
   private:
      MyPushButton *btn1;
      MyPushButton *btn2;
};

#include "moc_exemple10.h"

int main(int argc, char **argv)
{
   QApplication app(argc, argv);
   QWidget mainWidget;
   QHBoxLayout *horizontalLayout = new QHBoxLayout;
   horizontalLayout->addWidget(new MyFrame);
   horizontalLayout->addWidget(new MyFrame);

   mainWidget.setLayout(horizontalLayout);
   mainWidget.show();
   
   return app.exec();
}
