#include <QtGui>
#include <QString>

#include "shapedclock.h"

ShapedClock::ShapedClock(QWidget *parent)
    : QWidget(parent, Qt::FramelessWindowHint | Qt::WindowSystemMenuHint)
{
    drag = false;
    QAction *quitAction = new QAction(tr("E&xit"), this);
    quitAction->setShortcut(tr("Ctrl+Q"));
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
    addAction(quitAction);

    setContextMenuPolicy(Qt::ActionsContextMenu);
    setWindowTitle(tr("Qt by Nokia"));
}

void ShapedClock::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) 
    {
       dragPosition = event->globalPos() - frameGeometry().topLeft();
       event->accept();
    }
}

void ShapedClock::mouseMoveEvent(QMouseEvent *event)
{
    if (event->buttons() & Qt::LeftButton) 
    {   
       position = event->globalPos();
       drag = true;
       repaint(); //update();
       move(event->globalPos() - dragPosition);       
       event->accept();       
    }
}

void ShapedClock::mouseReleaseEvent(QMouseEvent *event)
{
   if(drag)
   {
      drag = false;   
      repaint();
   }
   event->accept();        
}

void ShapedClock::paintEvent(QPaintEvent *)
{
   QPainter painter(this);
   painter.setRenderHint(QPainter::Antialiasing);

   QRect rect(0, 0, 100, 100);

   painter.setPen( QPen(Qt::black, 2) );
   
   if(drag)
         painter.drawText(rect, Qt::AlignCenter, QString::number(position.x()) + "," + QString::number(position.y()));
   else  painter.drawText(rect, Qt::AlignCenter, "Qt by\nNokia\n\n");
}

void ShapedClock::resizeEvent(QResizeEvent * /* event */)
{
    int side = qMin(width(), height());
    QRegion maskedRegion(width() / 2 - side / 2, height() / 2 - side / 2, side, side, QRegion::Ellipse);
    setMask(maskedRegion);
}

QSize ShapedClock::sizeHint() const
{
    return QSize(100, 100);
}
