#include <QGridLayout>
#include <QTimer>
#include "speedo_meter.h"
#include "cockpit_grid.h"

typedef QPalette Palette;

CockpitGrid::CockpitGrid(QWidget *parent):
    QFrame(parent)
{
    setAutoFillBackground(true);

    //setPalette(colorTheme(QColor(Qt::darkGray).dark(150)));

    QGridLayout *layout = new QGridLayout(this);
    layout->setSpacing(5);
    layout->setMargin(0);

    QwtDial *dial = createDial();
    layout->addWidget(dial);
}

QwtDial *CockpitGrid::createDial()
{
    QwtDial *dial = NULL;
    
   d_speedo = new SpeedoMeter(this);
   d_speedo->setRange(0.0, 240.0);
   d_speedo->setScale(-1, 2, 20);

   QTimer *timer = new QTimer(d_speedo);
   timer->connect(timer, SIGNAL(timeout()), 
         this, SLOT(changeSpeed()));
   timer->start(50);

   dial = d_speedo;    

    if ( dial )
    {
        dial->setReadOnly(true);
        dial->scaleDraw()->setPenWidth(3);
        dial->setLineWidth(4);
        dial->setFrameShadow(QwtDial::Sunken);
    }
    return dial;
}

QPalette CockpitGrid::colorTheme(const QColor &base) const
{
    const QColor background = base.dark(150);
    const QColor foreground = base.dark(200);

    const QColor mid = base.dark(110);
    const QColor dark = base.dark(170);
    const QColor light = base.light(170);
    const QColor text = foreground.light(800);

    QPalette palette;
    for ( int i = 0; i < QPalette::NColorGroups; i++ )
    {
        QPalette::ColorGroup cg = (QPalette::ColorGroup)i;

        palette.setColor(cg, Palette::Base, base);
        palette.setColor(cg, Palette::Background, background);
        palette.setColor(cg, Palette::Mid, mid);
        palette.setColor(cg, Palette::Light, light);
        palette.setColor(cg, Palette::Dark, dark);
        palette.setColor(cg, Palette::Text, text);
        palette.setColor(cg, Palette::Foreground, foreground);
    }

    return palette;
}

void CockpitGrid::changeSpeed()
{
    static double offset = 0.8;

    double speed = d_speedo->value();

    if ( (speed < 40.0 && offset < 0.0 ) ||  
        (speed > 160.0 && offset > 0.0) )
    {
        offset = -offset;
    }

    static int counter = 0;
    switch(counter++ % 12 )
    {
        case 0:
        case 2:
        case 7:
        case 8:
            break;
        default:
            d_speedo->setValue(speed + offset);
    }
}
